/*
 * Flash_Access.c
 *
 *  Created on: 17. 2. 2021
 *      Author: Ondra
 */
#include "Flash_Access.h"

/* FLASH se maze s novym zapsanim, takze existuje jen zapis nove hodnoty nebo vycteni stare
 * Priklady pouziti
 * STRING DO PAMETI:
 * char write_data[50];	 //pole s daty
 * memset(write_data, 0, sizeof(write_data));	//vynulovani pole
 * strcpy(write_data, "Hello World!!!");	//zapsani Hello World do pole
 * save_to_flash((uint8_t*)write_data, strlen((char*)data)/8));	//zapsani dat do flash, spravne napocitat velikost bytu!!!
 *
 * POLE HODNOT DO PAMETI (BITY A NASTAVOVACI HODNOTY):
 * uint8_t write_data[3] = {0xAF,0x1D,0x0B};
 * save_to_flash((uint8_t*)write_data, 3);//spravne napocitat delku podle velikosti pole!!!
 *
 * CTENI DAT:
 * char read_data[50];	//pole pro ulozeni
 * memset(read_data, 0, sizeof(read_data)); //vynulovani
 * read_flash((uint8_t*)read_data);	//vycteni dat z flash funkci
 *
 * uint8_t var0, var1, var2;
 * var0= *(uint8_t*)(FLASH_STORAGE+0); //prime sahnuti do pameti pres pointery, elegantni, daji se brat i po vice bitech nez 8
 * var1= *(uint8_t*)(FLASH_STORAGE+1);
 * var2= *(uint8_t*)(FLASH_STORAGE+2);
 */


/*Moje verze funkce s pointery, pro obecne pouziti
 * vstup: pointer data na ulozena data, muze byt obecne i na vetsi promenne
 * vstup: promena length, rika kolik bytu budu zapisovat
 */


void save_to_flash(uint8_t *data, uint16_t length)
{
	/*Funkce zapise blok dat do pameti FLASH, a to od adresy 0x0807F800, page_size 0x800
	 *tzn velikost jedne stranky je 2048 bytu, vzdy se smaze cela stranka a prepise se obsah
	 *na jedno psani zapisuji 64bitu do flash pameti, zapisuji na maximalne jednu stranku, vic nechci
	 *pokud neni vstup nasobkem 8x bajtu, tak se zbytek doplni necim nahodnym z pameti (zapisuju 9 platnych, 7 je random a neresim je)
	 */
	volatile uint16_t write_times = 0;//spocte kolikrat musim zapsat do flash, zapisuju 8 bytu(64 bitu) naraz
	write_times = length /8;
	if(length%8!=0){
		write_times++;
	}

	if(write_times <= (page_size/8)){//vejde se na jednu stranku
		  /* Unlock the Flash to enable the flash control register access *************/
		  HAL_FLASH_Unlock();

		  /* Allow Access to option bytes sector */
		  HAL_FLASH_OB_Unlock();

		  /* Fill EraseInit structure*/
		  FLASH_EraseInitTypeDef EraseInitStruct;
		  EraseInitStruct.TypeErase = FLASH_TYPEERASE_PAGES;	//mazat stranku
		  EraseInitStruct.Banks = FLASH_BANK_1;	//v bance 1
		  EraseInitStruct.Page = 255;	//stranku cislo 255 - odpovida adresam 0x0807 F800 - 0x0807 FFFF viz RM str 97
		  EraseInitStruct.NbPages = 1;	//umazat jednu stranku
		  uint32_t PageError;

		  volatile uint32_t write_cnt=0, index=0;

		  volatile HAL_StatusTypeDef status;
		  status = HAL_FLASHEx_Erase(&EraseInitStruct, &PageError);	//smazat
		  while(index < write_times)
		  {
			  if (status == HAL_OK)
			  {
				  status = HAL_FLASH_Program(FLASH_TYPEPROGRAM_DOUBLEWORD, FLASH_STORAGE+write_cnt, *((uint64_t*)(data + write_cnt)));
				  if(status == HAL_OK)
				  {
					  write_cnt += 8;//pro adresaci, ukazuje adresu po osmi
					  index++;
				  }
			  }
		  }

		  HAL_FLASH_OB_Lock();
		  HAL_FLASH_Lock();

	}
	else{
		;//nic, nezapisu
	}


}


/* Stara verze funkce podle navodu, moc slozita
 * https://www.youtube.com/watch?v=5GhHpeRgceM&ab_channel=ViktorVano
 */
void save_to_flash_old(uint8_t *data)
{
	volatile uint64_t data_to_FLASH[(strlen((char*)data)/8)	+ (int)((strlen((char*)data) % 8) != 0)];
	memset((uint8_t*)data_to_FLASH, 0, strlen((char*)data_to_FLASH));
	strcpy((char*)data_to_FLASH, (char*)data);

	volatile uint64_t data_length = (strlen((char*)data_to_FLASH) / 8)
									+ (int)((strlen((char*)data_to_FLASH) % 8) != 0);
	volatile uint16_t pages = (strlen((char*)data)/page_size)
									+ (int)((strlen((char*)data)%page_size) != 0);
	  /* Unlock the Flash to enable the flash control register access *************/
	  HAL_FLASH_Unlock();

	  /* Allow Access to option bytes sector */
	  HAL_FLASH_OB_Unlock();


	  /* Fill EraseInit structure*/
	  FLASH_EraseInitTypeDef EraseInitStruct;
	  EraseInitStruct.TypeErase = FLASH_TYPEERASE_PAGES;
	  EraseInitStruct.Banks = FLASH_BANK_1;
	  EraseInitStruct.Page = 255;
	  EraseInitStruct.NbPages = pages;
	  uint32_t PageError;


	  volatile uint32_t write_cnt=0, index=0;

	  volatile HAL_StatusTypeDef status;
	  status = HAL_FLASHEx_Erase(&EraseInitStruct, &PageError);
	  while(index < data_length)
	  {
		  if (status == HAL_OK)
		  {
			  status = HAL_FLASH_Program(FLASH_TYPEPROGRAM_DOUBLEWORD, FLASH_STORAGE+write_cnt, data_to_FLASH[index]);
			  if(status == HAL_OK)
			  {
				  write_cnt += 8;//bylo + 4, ale zapisuju dword
				  index++;
			  }
		  }
	  }

	  HAL_FLASH_OB_Lock();
	  HAL_FLASH_Lock();
}


/* Precte data z flashky, podle adresy dane definem
 * Nic extra, sp pro pedstavu jak to funguje
 */
void read_flash(uint8_t* data)
{
	volatile uint32_t read_data;
	volatile uint32_t read_cnt=0;
	do
	{
		read_data = *(uint32_t*)(FLASH_STORAGE + read_cnt);
		if(read_data != 0xFFFFFFFF)
		{
			data[read_cnt] = (uint8_t)read_data;
			data[read_cnt + 1] = (uint8_t)(read_data >> 8);
			data[read_cnt + 2] = (uint8_t)(read_data >> 16);
			data[read_cnt + 3] = (uint8_t)(read_data >> 24);
			read_cnt += 4;
		}
	}while(read_data != 0xFFFFFFFF);
}

uint8_t read_flash_byte(uint32_t address){
	return (uint8_t)(*(uint32_t*)(address));
}
