/*
 * SMBusLib.c
 *
 *  Created on: 23. 11. 2020
 *      Author: Ondra
 */
#include "stm32l4xx.h"
#include "SMBusLib.h"
#include "main.h"
#include "stdio.h"
#include "string.h"
#include "CAN.h"

extern I2C_HandleTypeDef hi2c1;
extern UART_HandleTypeDef huart2;



/*
 * Poslani WORDu po I2C (SMBUS), odzkouseno na BQ24725A, funguje dobre
 * Vraci HAL_Status - HAL_OK pokud je zarizeni dostupne a povede se vse, testovat pred volanim !=HAL_OK tak ....
 */
HAL_StatusTypeDef SMBus_Write_Word(uint8_t address, uint8_t command, uint16_t data) {
	uint8_t CAN_msg = 0;
	uint8_t Hal_Err;
	uint8_t i2cData[3];
	i2cData[0] = command;
	i2cData[1] = (uint8_t) data; //LOW data byte
	i2cData[2] = (uint8_t) (data >> 8); //HIGH data byte

	Hal_Err = HAL_I2C_Master_Transmit(&hi2c1, address, &i2cData[0], 3, 10); //command + data = 3 bytes!!
	//tahle funkce je blokujici, ceka se na odeslani celeho wordu
	if (Hal_Err != HAL_OK) {
		CAN_msg = Error_type_SMBus;
		CAN_Send_Message(ID_Error, 1, &CAN_msg);
	}
	return Hal_Err;
}

uint16_t SMBus_Read_Word(uint8_t address, uint8_t command) {
	uint8_t CAN_msg = 0;
	uint8_t i2cData[2];	//prichozi word
	uint16_t rec_data = 0;	//prohodit poradi bytu, posila low byte jako prvni, highbyte jako druhy
	//poslat command, generovat restart
	if (HAL_I2C_Master_Seq_Transmit_IT(&hi2c1, address, (uint8_t *) &command, 1,
	I2C_FIRST_FRAME) != HAL_OK) {
		Error_Handler();/* Error_Handler() function is called when error occurs. */
	}
	/*##-3- Wait for the end of the transfer #################################*/
	while (HAL_I2C_GetState(&hi2c1) != HAL_I2C_STATE_READY) {
	}
	/* When Acknowledge failure occurs (Slave don't acknowledge it's address)
	 Master restarts communication */
	if (HAL_I2C_GetError(&hi2c1) == HAL_I2C_ERROR_AF) {
		CAN_msg = Error_type_SMBus;
		CAN_Send_Message(ID_Error, 1, &CAN_msg);
		return 0xFFFF;	//ACK error
	}
	/*##-4- Put I2C peripheral in reception process ############################*/
	//prijmout 2 bytes aulozit do i2cData
	if (HAL_I2C_Master_Seq_Receive_IT(&hi2c1, address, (uint8_t *) &i2cData, 2,
	I2C_LAST_FRAME) != HAL_OK) {
		Error_Handler();/* Error_Handler() function is called when error occurs. */
	}

	/*##-5- Wait for the end of the transfer #################################*/
	while (HAL_I2C_GetState(&hi2c1) != HAL_I2C_STATE_READY) {
	}
	/* When Acknowledge failure occurs (Slave don't acknowledge it's address)
	 Master restarts communication */
	if (HAL_I2C_GetError(&hi2c1) == HAL_I2C_ERROR_AF) {
		CAN_msg = Error_type_SMBus;
		CAN_Send_Message(ID_Error, 1, &CAN_msg);
		return 0xFFFF;	//ACK error
	}
	rec_data = ((i2cData[1] << 8) | i2cData[0]);
	return rec_data;	//vracet i CHYBU
}

uint16_t SMBus_Read_Word_old(uint8_t address, uint8_t command) {

	//funkce ukoncene s IT generuji interrupt - skoci do filu stm32l4xx_it.c funkce, tady ignoruju, cekam na konec blokovanim
	uint8_t i2cData[2];	//prichozi word
	uint16_t rec_data = 0;	//prohodit poradi bytu, posila low byte jako prvni, highbyte jako druhy
	do {	//poslat command, generovat restart
		if (HAL_I2C_Master_Seq_Transmit_IT(&hi2c1, address, (uint8_t *) &command, 1, I2C_FIRST_FRAME) != HAL_OK) {
			Error_Handler();/* Error_Handler() function is called when error occurs. */
		}
		/*##-3- Wait for the end of the transfer #################################*/
		while (HAL_I2C_GetState(&hi2c1) != HAL_I2C_STATE_READY) {
		}
		/* When Acknowledge failure occurs (Slave don't acknowledge it's address)
		 Master restarts communication */
	} while (HAL_I2C_GetError(&hi2c1) == HAL_I2C_ERROR_AF);
	/*##-4- Put I2C peripheral in reception process ############################*/
	do {	//prijmout 2 bytes aulozit do i2cData
		if (HAL_I2C_Master_Seq_Receive_IT(&hi2c1, address, (uint8_t *) &i2cData, 2, I2C_LAST_FRAME) != HAL_OK) {
			Error_Handler();/* Error_Handler() function is called when error occurs. */
		}

		/*##-5- Wait for the end of the transfer #################################*/
		while (HAL_I2C_GetState(&hi2c1) != HAL_I2C_STATE_READY) {
		}
		/* When Acknowledge failure occurs (Slave don't acknowledge it's address)
		 Master restarts communication */
	} while (HAL_I2C_GetError(&hi2c1) == HAL_I2C_ERROR_AF);

	rec_data = ((i2cData[1] << 8) | i2cData[0]);
	return rec_data;	//vracet i CHYBU
}

uint8_t SMBus_Read_Block(uint8_t address, uint8_t command, uint8_t *pData) {
	uint8_t CAN_msg = 0;
	//precte blok po smbusu
	//pdata - oblast kam se bude zapisovat
	uint8_t block_len = 0;	//delka prijateho blocku
	//POSLAT ADRESU A COMMAND - vysilam
	if (HAL_I2C_Master_Seq_Transmit_IT(&hi2c1, address, (uint8_t *) &command, 1,
	I2C_FIRST_FRAME) != HAL_OK) {
		Error_Handler();/* Error_Handler() function is called when error occurs. */
	}
	/*##-3- Wait for the end of the transfer #################################*/
	while (HAL_I2C_GetState(&hi2c1) != HAL_I2C_STATE_READY) {
	}
	/* When Acknowledge failure occurs (Slave don't acknowledge it's address)
	 Master restarts communication */
	if (HAL_I2C_GetError(&hi2c1) == HAL_I2C_ERROR_AF) {
		CAN_msg = Error_type_SMBus;
		CAN_Send_Message(ID_Error, 1, &CAN_msg);
		return 0xFF;	//ACK error
	}

	//PRIJIMAM 1 byte, Zmena smeru generuje sama restart, Zjistuju DELKU blocku podle 1.bytu v prijmu
	if (HAL_I2C_Master_Seq_Receive_IT(&hi2c1, address, (uint8_t *) &block_len, 1, I2C_NEXT_FRAME) != HAL_OK) {
		Error_Handler();/* Error_Handler() function is called when error occurs. */
	}

	/*##-5- Wait for the end of the transfer #################################*/
	while (HAL_I2C_GetState(&hi2c1) != HAL_I2C_STATE_READY) {
	}
	//blok by nemel byt 0 nebo delsi nez 32 bytu, z pameti leze 34

	if (block_len == 0 || block_len > 34) {		//chyba
		HAL_I2C_Master_Abort_IT(&hi2c1, address);
		CAN_msg = Error_type_SMBus;
		CAN_Send_Message(ID_Error, 1, &CAN_msg);
		return 0xFF;
	}

	//POKRACUJU v Prijmu, prijimam X bytu podle 1.bytu viz vyse
	/*##-4- Put I2C peripheral in reception process ############################*/
	if (HAL_I2C_Master_Seq_Receive_IT(&hi2c1, address, pData, block_len,
	I2C_LAST_FRAME) != HAL_OK) {
		Error_Handler();/* Error_Handler() function is called when error occurs. */
	}

	/*##-5- Wait for the end of the transfer #################################*/
	while (HAL_I2C_GetState(&hi2c1) != HAL_I2C_STATE_READY) {
	}
	/* When Acknowledge failure occurs (Slave don't acknowledge it's address)
	 Master restarts communication */
	if (HAL_I2C_GetError(&hi2c1) == HAL_I2C_ERROR_AF) {
		CAN_msg = Error_type_SMBus;
		CAN_Send_Message(ID_Error, 1, &CAN_msg);
		return 0xFF;		//ACK error
	}
//tady vycist co prijde pres debug
	return block_len;
}

HAL_StatusTypeDef SMBus_Write_Block(uint8_t address, uint8_t command, uint8_t *pData, uint8_t data_len) {
	uint8_t CAN_msg = 0;
	if (data_len == 0 || data_len > 34) {
		CAN_msg = Error_type_SMBus;
		CAN_Send_Message(ID_Error, 1, &CAN_msg);
		return HAL_ERROR;		//moc dlouhy data na SMBus, z pameti leze 34
	}

	//POSLAT ADRESU A COMMAND - vysilam
	if (HAL_I2C_Master_Seq_Transmit_IT(&hi2c1, address, (uint8_t *) &command, 1,
	I2C_FIRST_AND_NEXT_FRAME) != HAL_OK) {
		Error_Handler();/* Error_Handler() function is called when error occurs. */
	}
	/*##-3- Wait for the end of the transfer #################################*/
	while (HAL_I2C_GetState(&hi2c1) != HAL_I2C_STATE_READY) {
	}
	/* When Acknowledge failure occurs (Slave don't acknowledge it's address)
	 Master restarts communication */

	if (HAL_I2C_GetError(&hi2c1) == HAL_I2C_ERROR_AF) {
		CAN_msg = Error_type_SMBus;
		CAN_Send_Message(ID_Error, 1, &CAN_msg);
		return HAL_ERROR;
	}

	//POSLAT BYTE COUNT
	if (HAL_I2C_Master_Seq_Transmit_IT(&hi2c1, address, (uint8_t *) &data_len, 1, I2C_NEXT_FRAME) != HAL_OK) {
		Error_Handler();/* Error_Handler() function is called when error occurs. */
	}
	/*##-3- Wait for the end of the transfer #################################*/
	while (HAL_I2C_GetState(&hi2c1) != HAL_I2C_STATE_READY) {
	}
	//POSLAT DATA
	if (HAL_I2C_Master_Seq_Transmit_IT(&hi2c1, address, pData, data_len,
	I2C_LAST_FRAME) != HAL_OK) {
		CAN_msg = Error_type_SMBus;
		CAN_Send_Message(ID_Error, 1, &CAN_msg);
		Error_Handler();/* Error_Handler() function is called when error occurs. */
	}
	/*##-3- Wait for the end of the transfer #################################*/
	while (HAL_I2C_GetState(&hi2c1) != HAL_I2C_STATE_READY) {
	}

	return HAL_OK;
	//return HAL_I2C_Master_Transmit(&hi2c1, address, pData, data_len + 2, 10); //command + byte count + data samotna
	//tahle funkce je blokujici, ceka se na odeslani celeho blocku
}

void BQ24725A_test(void) {
	uint16_t SMB_data[6];


	if (HAL_I2C_IsDeviceReady(&hi2c1, BQ24725A_Address, 2, 10) == HAL_OK)	//, handler, adresa 0x12, pocet pokusu,timeout
			{
		HAL_GPIO_WritePin(LD2_GPIO_Port, LD2_Pin, GPIO_PIN_SET);	//ok zarizeni odpovida,sviti led
	} else {
		HAL_GPIO_WritePin(LD2_GPIO_Port, LD2_Pin, GPIO_PIN_RESET);	//chyba, zarizeni neodpovida
	}

	SMB_data[0] = SMBus_Read_Word(BQ24725A_Address, BQ24725A_ChargeOption);
	SMB_data[1] = SMBus_Read_Word(BQ24725A_Address, BQ24725A_ChargeCurrent);
	SMB_data[2] = SMBus_Read_Word(BQ24725A_Address, BQ24725A_ChargeVoltage);
	SMB_data[3] = SMBus_Read_Word(BQ24725A_Address, BQ24725A_InputCurrent);
	SMB_data[4] = SMBus_Read_Word(BQ24725A_Address, BQ24725A_ManufacturerID);
	SMB_data[5] = SMBus_Read_Word(BQ24725A_Address, BQ24725A_DeviceID);

	SMBus_Write_Word(BQ24725A_Address, BQ24725A_ChargeOption, SMB_data[0] | 0x0001 | 0x0020);	//disable charging, Iout
	SMB_data[0] = SMBus_Read_Word(BQ24725A_Address, BQ24725A_ChargeOption);

	if (0) {
		//mala baterka, 1 do ifu; velka baterka - 0 do ifu
		//SMBus_Write_Word(BQ24725A_Address, BQ24725A_InputCurrent, 0x0800);//2048mA input current
		SMBus_Write_Word(BQ24725A_Address, BQ24725A_InputCurrent, 0x1F80);		//8064mA input current
		SMB_data[3] = SMBus_Read_Word(BQ24725A_Address, BQ24725A_InputCurrent);
		SMBus_Write_Word(BQ24725A_Address, BQ24725A_ChargeCurrent, 0x0100);	//256mA charging current
		SMB_data[1] = SMBus_Read_Word(BQ24725A_Address, BQ24725A_ChargeCurrent);
		SMBus_Write_Word(BQ24725A_Address, BQ24725A_ChargeVoltage, 0x3130);	//12592mV charging voltage
		//SMBus_Write_Word(BQ24725A_Address, BQ24725A_ChargeVoltage, 0x38B0);	//14512mV charging voltage
		SMB_data[2] = SMBus_Read_Word(BQ24725A_Address, BQ24725A_ChargeVoltage);

		SMBus_Write_Word(BQ24725A_Address, BQ24725A_ChargeOption, SMB_data[0] & 0xFFFE);	//enable charging
		SMB_data[0] = SMBus_Read_Word(BQ24725A_Address, BQ24725A_ChargeOption);

		SMBus_Write_Word(BQ24725A_Address, BQ24725A_ChargeCurrent, 0x0200);	//512mA charging current
		SMB_data[1] = SMBus_Read_Word(BQ24725A_Address, BQ24725A_ChargeCurrent);
		SMBus_Write_Word(BQ24725A_Address, BQ24725A_ChargeCurrent, 0x0300);	//768mA charging current
		SMB_data[1] = SMBus_Read_Word(BQ24725A_Address, BQ24725A_ChargeCurrent);
		SMBus_Write_Word(BQ24725A_Address, BQ24725A_ChargeCurrent, 0x0400);	//1024mA charging current
		SMB_data[1] = SMBus_Read_Word(BQ24725A_Address, BQ24725A_ChargeCurrent);

		SMBus_Write_Word(BQ24725A_Address, BQ24725A_ChargeOption, SMB_data[0] | 0x0001);	//disable charging
		SMB_data[0] = SMBus_Read_Word(BQ24725A_Address, BQ24725A_ChargeOption);

	} else {
		//velka baterka

		SMBus_Write_Word(BQ24725A_Address, BQ24725A_InputCurrent, 0x1F80);	//8064mA input current
		SMB_data[3] = SMBus_Read_Word(BQ24725A_Address, BQ24725A_InputCurrent);
		SMBus_Write_Word(BQ24725A_Address, BQ24725A_ChargeCurrent, 0x0100);	//256mA charging current
		SMB_data[1] = SMBus_Read_Word(BQ24725A_Address, BQ24725A_ChargeCurrent);

		SMBus_Write_Word(BQ24725A_Address, BQ24725A_ChargeVoltage, 0x3130);	//12592mV charging voltage
		//SMBus_Write_Word(BQ24725A_Address, BQ24725A_ChargeVoltage, 0x38B0);	//14512mV charging voltage
		SMB_data[2] = SMBus_Read_Word(BQ24725A_Address, BQ24725A_ChargeVoltage);

		SMBus_Write_Word(BQ24725A_Address, BQ24725A_ChargeOption, SMB_data[0] & 0xFFFE);	//enable charging
		SMB_data[0] = SMBus_Read_Word(BQ24725A_Address, BQ24725A_ChargeOption);
		/*
		 SMBus_Write_Word(BQ24725A_Address, BQ24725A_ChargeCurrent, 0x0400);	//1024mA charging current
		 SMB_data[1] = SMBus_Read_Word(BQ24725A_Address, BQ24725A_ChargeCurrent);
		 SMBus_Write_Word(BQ24725A_Address, BQ24725A_ChargeCurrent, 0x0800);	//2048mA charging current
		 SMB_data[1] = SMBus_Read_Word(BQ24725A_Address, BQ24725A_ChargeCurrent);

		 SMBus_Write_Word(BQ24725A_Address, BQ24725A_ChargeCurrent, 0x0C00);	//3072mA charging current
		 SMB_data[1] = SMBus_Read_Word(BQ24725A_Address, BQ24725A_ChargeCurrent);

		 SMBus_Write_Word(BQ24725A_Address, BQ24725A_ChargeCurrent, 0x1000);	//4096mA charging current
		 SMB_data[1] = SMBus_Read_Word(BQ24725A_Address, BQ24725A_ChargeCurrent);
		 */
		SMBus_Write_Word(BQ24725A_Address, BQ24725A_ChargeCurrent, 0x1400);	//5120mA charging current
		SMB_data[1] = SMBus_Read_Word(BQ24725A_Address, BQ24725A_ChargeCurrent);

		SMBus_Write_Word(BQ24725A_Address, BQ24725A_ChargeCurrent, 0x1800);	//6144mA charging current
		SMB_data[1] = SMBus_Read_Word(BQ24725A_Address, BQ24725A_ChargeCurrent);

		SMBus_Write_Word(BQ24725A_Address, BQ24725A_ChargeCurrent, 0x1C00);	//7168mA charging current
		SMB_data[1] = SMBus_Read_Word(BQ24725A_Address, BQ24725A_ChargeCurrent);

		SMBus_Write_Word(BQ24725A_Address, BQ24725A_ChargeCurrent, 0x1FC0);	//8128mA charging current
		SMB_data[1] = SMBus_Read_Word(BQ24725A_Address, BQ24725A_ChargeCurrent);

		SMBus_Write_Word(BQ24725A_Address, BQ24725A_ChargeOption, SMB_data[0] | 0x0001);	//disable charging
		SMB_data[0] = SMBus_Read_Word(BQ24725A_Address, BQ24725A_ChargeOption);
	}

	SMBus_Write_Word(BQ24725A_Address, BQ24725A_ChargeOption, SMB_data[0] | 0x0001);	//disable charging
	SMB_data[0] = SMBus_Read_Word(BQ24725A_Address, BQ24725A_ChargeOption);

}

void BQ40Z80_test() {
	if (HAL_I2C_IsDeviceReady(&hi2c1, BQ40Z80_Address, 2, 10) == HAL_OK) //, handler, adresa 0x16, pocet pokusu,timeout
			{
		HAL_GPIO_WritePin(LD2_GPIO_Port, LD2_Pin, GPIO_PIN_SET); //ok zarizeni odpovida
	} else {
		HAL_GPIO_WritePin(LD2_GPIO_Port, LD2_Pin, GPIO_PIN_RESET); //chyba, zarizeni neodpovida
	}
	uint8_t SMB_data[36];
	uint8_t SMB_data2[36];
	uint8_t data_length = 0;
	uint16_t voltage = 0;
	uint16_t batterymode = 0;

	voltage = SMBus_Read_Word(BQ40Z80_Address, 0x09); //voltage

	//SMBus_Write_Word(BQ40Z80_Address, 0x03, 0x6001);
	//batterymode = SMBus_Read_Word(BQ40Z80_Address, 0x03); //batterymode

	data_length = SMBus_Read_Block(BQ40Z80_Address, 0x21, &SMB_data[0]); //device name BQ40Z80

	//read chem id
	uint8_t ChemID[2] = { 0x06, 0x00 };
	SMBus_Write_Block(BQ40Z80_Address, 0x44, &ChemID[0], 2);
	data_length = SMBus_Read_Block(BQ40Z80_Address, 0x44, &SMB_data[0]);

	//read & write enabled protections A
	//uint8_t FlashAddresa[2] = { 0xBE, 0x4B }; //cist
	//SMBus_Write_Block(BQ40Z80_Address, 0x44, &FlashAddresa[0], 2);
	//data_length = SMBus_Read_Block(BQ40Z80_Address, 0x44, &SMB_data[0]);

	//uint8_t FlashDatka[3] = { 0xBE, 0x4B, 0x03 }; //zapsat enabled CUV a COV
	//SMBus_Write_Block(BQ40Z80_Address, 0x44, &FlashDatka[0], 3);

	//SMBus_Write_Block(BQ40Z80_Address, 0x44, &FlashAddresa[0], 2); //overit ze je zapsano
	//data_length = SMBus_Read_Block(BQ40Z80_Address, 0x44, &SMB_data[0]);

	//BQ40Z80_Flash_Read_Block(0x4AD3); //pin config
	//BQ40Z80_Flash_Read_All();

}

void BQ40Z80_Flash_Read_All(void) {
	//cteni FLASHky, precte celou flashku a vyblinka ji do puttyho, d se pak stahnout do PC
	uint8_t FlashAddress[2] = { 0x00, 0x40 };	//adresa
	uint8_t SMB_databig[40];
	uint16_t BQMem[32][2];	//32 radku, 2 sloupce -1 sloup adresa, 2 sloup - byte
	uint8_t i = 0;
	uint8_t exit = 0;
	uint16_t TempAddress = 0;
	char buf[30];
	uint8_t data_length = 0;

	SMBus_Write_Block(BQ40Z80_Address, 0x44, &FlashAddress[0], 2);	//a. Send SMBus write block with command 0x44, block = 0x00 + 0x40
	while (1) {
		data_length = SMBus_Read_Block(BQ40Z80_Address, 0x44, &SMB_databig[0]);
		TempAddress = 0;
		TempAddress = ((SMB_databig[0]) | (SMB_databig[1] << 8));
		for (i = 0; i < 32; i++) {
			BQMem[i][0] = TempAddress + i;	//i ty radek a prvni(0) sloupec
			BQMem[i][1] = SMB_databig[i + 2];	//i ty radek, druhy(1) sloupec, data 2-34 protoze prvni prijde adresa
			sprintf(buf, "0x%04X 0x%02X\r\n", TempAddress + i, SMB_databig[i + 2]);
			HAL_UART_Transmit(&huart2, &buf[0], strlen(&buf[0]), 1000);
			if (TempAddress + i >= 0x5FFF) {
				exit = 1;
				break;
			}
		}
		if (exit) {
			break;
		}

	}

}

void BQ40Z80_Flash_Read_Block(uint16_t flashaddr) {
	//cteni FLASHky, precte 32 bytu z flashky, jinak to nejde, vraci 34 bytu - start adresa (2 bytes) a data
	//vysledek putty, nebo odkomentovat bqmem a vycist si to z toho
	uint8_t FlashAddress[2] = { 0x00, 0x00 };	//adresa
	FlashAddress[0] = (uint8_t) flashaddr;
	FlashAddress[1] = (uint8_t) (flashaddr >> 8);
	uint8_t SMB_databig[40];
	//uint16_t BQMem[32][2];//32 radku, 2 sloupce -1 sloup adresa, 2 sloup - byte
	uint8_t i = 0;
	uint8_t data_length = 0;
	uint16_t TempAddress = 0;
	char buf[30];
	SMBus_Write_Block(BQ40Z80_Address, 0x44, &FlashAddress[0], 2);	//a. Send SMBus write block with command 0x44, block = 0x00 + 0x40

	data_length = SMBus_Read_Block(BQ40Z80_Address, 0x44, &SMB_databig[0]);
	TempAddress = ((SMB_databig[0]) | (SMB_databig[1] << 8));
	for (i = 0; i < 32; i++) {
		//BQMem[i][0] = TempAddress + i;	//i ty radek a prvni(0) sloupec
		//BQMem[i][1] = SMB_databig[i + 2];//i ty radek, druhy(1) sloupec, data 2-34 protoze prvni prijde adresa
		sprintf(buf, "0x%04X 0x%02X\r\n", TempAddress + i, SMB_databig[i + 2]);
		HAL_UART_Transmit(&huart2, &buf[0], strlen(&buf[0]), 1000);
		if (TempAddress + i >= 0x5FFF) {
			break;
		}
	}
}
