/* USER CODE BEGIN Header */
/**
 ******************************************************************************
 * @file           : main.c
 * @brief          : Main program body
 ******************************************************************************
 * @attention
 *
 * <h2><center>&copy; Copyright (c) 2021 STMicroelectronics.
 * All rights reserved.</center></h2>
 *
 * This software component is licensed by ST under BSD 3-Clause license,
 * the "License"; You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at:
 *                        opensource.org/licenses/BSD-3-Clause
 *
 ******************************************************************************
 */
/* USER CODE END Header */
/* Includes ------------------------------------------------------------------*/
#include "main.h"

/* Private includes ----------------------------------------------------------*/
/* USER CODE BEGIN Includes */
#include "CAN.h"
#include "SMBusLib.h"
#include "stdio.h"
#include "string.h"
#include "Flash_Access.h"
/* USER CODE END Includes */

/* Private typedef -----------------------------------------------------------*/
/* USER CODE BEGIN PTD */

/* USER CODE END PTD */

/* Private define ------------------------------------------------------------*/
/* USER CODE BEGIN PD */
/* USER CODE END PD */

/* Private macro -------------------------------------------------------------*/
/* USER CODE BEGIN PM */

/* USER CODE END PM */

/* Private variables ---------------------------------------------------------*/
ADC_HandleTypeDef hadc1;
ADC_HandleTypeDef hadc2;

CAN_HandleTypeDef hcan1;

I2C_HandleTypeDef hi2c1;

UART_HandleTypeDef huart2;

/* USER CODE BEGIN PV */
CAN_TxHeaderTypeDef pTxHeader;
CAN_RxHeaderTypeDef pRxHeader; //Header prijate zpravy, obsahuje data o frame (ID, DLC, FilterMatchIndex...)
uint32_t TxMailbox;			//mailbox number
CAN_FilterTypeDef sFilterConfig;
uint8_t CAN_Rx_Buffer[8] = { 0 };
uint8_t CAN_Tx_Buffer[8] = { 0 };

uint16_t memChargeCurrent = 0;	//promenne nacitane z pameti flash
uint16_t memChargeVoltage = 0;
uint16_t memInputCurrent = 0;
uint16_t memChargeOption = 0;
uint8_t memBatteryPeriod = 0;

/* USER CODE END PV */

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);
static void MX_GPIO_Init(void);
static void MX_CAN1_Init(void);
static void MX_I2C1_Init(void);
static void MX_USART2_UART_Init(void);
static void MX_ADC1_Init(void);
static void MX_ADC2_Init(void);
/* USER CODE BEGIN PFP */

/* USER CODE END PFP */

/* Private user code ---------------------------------------------------------*/
/* USER CODE BEGIN 0 */
void HAL_I2C_MasterTxCpltCallback(I2C_HandleTypeDef *hi2c) {
	asm("nop");
}
/* USER CODE END 0 */

/**
 * @brief  The application entry point.
 * @retval int
 */
int main(void) {
	/* USER CODE BEGIN 1 */

	/* USER CODE END 1 */

	/* MCU Configuration--------------------------------------------------------*/

	/* Reset of all peripherals, Initializes the Flash interface and the Systick. */
	HAL_Init();

	/* USER CODE BEGIN Init */

	/* USER CODE END Init */

	/* Configure the system clock */
	SystemClock_Config();

	/* USER CODE BEGIN SysInit */

	/* USER CODE END SysInit */

	/* Initialize all configured peripherals */
	MX_GPIO_Init();
	MX_CAN1_Init();
	MX_I2C1_Init();
	MX_USART2_UART_Init();
	MX_ADC1_Init();
	MX_ADC2_Init();
	/* USER CODE BEGIN 2 */

	//TODO: !!!V CUBE se da nastavit loopback na canu!!!
	//FLASH pamet, ulozene hodnoty pro BQ24725A - 4x 16 bit, registry ChargeCurrent,ChargeVoltage,InputCurrent, ChargeOption v tomto poradi +
	//interval odesilani dat
	//ChargeCurrent =  0x0x1800 (6144mA),ChargeVoltage 0x3130(12592mV),InputCurrent 0x1F80 (8064mA), ChargeOption 0xF902(vychozi z DS)
	//Vychozi interval pro odesilani info o akumulatoru 0x0A (10x100ms)
	if (!HAL_GPIO_ReadPin(B1_GPIO_Port, B1_Pin)) {	//LOW je zmackle TLACITKO
		HAL_Delay(2000);
		if (!HAL_GPIO_ReadPin(B1_GPIO_Port, B1_Pin)) {	//je zmackle i po dvou vterinach, nahraju vychozi nastaveni
			uint8_t write_data[9] = { 0x00, 0x18, 0x30, 0x31, 0x80, 0x1F, 0x02, 0xF9, 0x0A };
			save_to_flash((uint8_t*) write_data, 9);
			for (int i = 0; i < 3; i++) {
				HAL_GPIO_WritePin(LD2_GPIO_Port, LD2_Pin, GPIO_PIN_SET);
				HAL_Delay(100);
				HAL_GPIO_WritePin(LD2_GPIO_Port, LD2_Pin, GPIO_PIN_RESET);
				HAL_Delay(100);
			}

		}
	}

	HAL_GPIO_WritePin(EN_GPIO_Port, EN_Pin, GPIO_PIN_SET);	//EN HIGH - BQ24725A enable - kdyz chci komunikovat s nabijeckou a bezi to z baterky
	HAL_GPIO_WritePin(RS_GPIO_Port, RS_Pin, GPIO_PIN_RESET);	//RS LOW - slope control, can transceiver, high speed mode
	HAL_GPIO_WritePin(CAN_EN_GPIO_Port, CAN_EN_Pin, GPIO_PIN_SET);	//CAN EN HIGH - wake up can transceiver

	HAL_Delay(10); //1-2ms to wake for SMBUS communication (BQ40Z80)
	memChargeCurrent = *(uint16_t*) (FLASH_STORAGE + 0); //prime sahnuti do pameti pres pointery, daji se brat i po vice bitech nez 8
	memChargeVoltage = *(uint16_t*) (FLASH_STORAGE + 2); //viz organizace flash vyse
	memInputCurrent = *(uint16_t*) (FLASH_STORAGE + 4);
	memChargeOption = *(uint16_t*) (FLASH_STORAGE + 6);
	memBatteryPeriod = *(uint8_t*) (FLASH_STORAGE + 8);

	if (HAL_I2C_IsDeviceReady(&hi2c1, BQ40Z80_Address, 10, 100) == HAL_OK)	//je dostupne BQ40Z80? Jinak kratce pripojit nabijecku->wake up
			{
		//ok, odpovida
	} else {

		SMBus_Write_Word(BQ24725A_Address, BQ24725A_InputCurrent, 0x1F80);	//8064mA input current
		SMBus_Write_Word(BQ24725A_Address, BQ24725A_ChargeVoltage, 0x3130);	//12592mV charging voltage
		SMBus_Write_Word(BQ24725A_Address, BQ24725A_ChargeCurrent, 0x1000);	//4096mA charging current
		SMBus_Write_Word(BQ24725A_Address, BQ24725A_ChargeOption, 0xF902);	//enable charging
		HAL_Delay(1000);	//1000ms impuls na probuzeni BQ40Z80
		SMBus_Write_Word(BQ24725A_Address, BQ24725A_ChargeVoltage, 0x0000);	//0mV charging voltage, zastavi nabijeni
		SMBus_Write_Word(BQ24725A_Address, BQ24725A_ChargeCurrent, 0x0000);	//0mA charging current, zastavi nabijeni
	}

	SMBus_Write_Word(BQ24725A_Address, BQ24725A_InputCurrent, memInputCurrent);
	SMBus_Write_Word(BQ24725A_Address, BQ24725A_ChargeOption, memChargeOption);

	CAN_Filter_init();
	HAL_CAN_Start(&hcan1);
	//uint8_t uartdata[] = { "Start App\n\r" };
	//HAL_UART_Transmit(&huart2, &uartdata[0], 11, 1000);
	/* USER CODE END 2 */

	/* Infinite loop */
	/* USER CODE BEGIN WHILE */
	uint32_t time1 = 0, time2 = 0, time3 = 0, time4 = 0; //timestampy pro casovani se systickem
	uint8_t buttonblok = 1;		//blokovani tlacitka
	uint8_t onetimeonly = 1;	//provest jen jednou
	uint8_t chg_enable = 1;	//po zapnuti umoznim nabijeni
	uint16_t batterystatus = 0;	//nacitani registru battery status z B40Z80
	uint16_t chargeoption = 0;	//nacteni charge option z BQ24725A
	uint32_t timer24h = 0;	//24 hour timer konec nabijeni. 24h = 86400sec =>8640*10 sec
	uint16_t ReadWordBuf[3] = { 0 };	//nacteni hodnot z BQ40Z80

	while (1) {
		if (HAL_GetTick() > time1) {	//500ms smyka
			time1 = HAL_GetTick() + 500;
			HAL_GPIO_TogglePin(LD2_GPIO_Port, LD2_Pin);	//jen blikani ledkou
		}

		if ((HAL_GetTick() > time2) && 1) {	//1000ms smyka
			time2 = HAL_GetTick() + 1000;
			batterystatus = SMBus_Read_Word(BQ40Z80_Address, battery_status_reg); //precteni registru batterystatus
			if ((batterystatus & battery_status_FC) || (batterystatus & battery_status_TCA) || (timer24h > 8640)) {	//aktivni FC nebo TCA, prestat nabijet
				chg_enable = 0;	//zastavit nabijeni
				if (onetimeonly) {	//vypne dobijeni, provede se pouze jednou, promenna onetimeonly se vynuluje a uz nikdy se nedostane do 1
					chargeoption = SMBus_Read_Word(BQ24725A_Address, BQ24725A_ChargeOption);
					SMBus_Write_Word(BQ24725A_Address, BQ24725A_ChargeOption, chargeoption | charge_option_charge_inhibit);	//inhibit charge
					onetimeonly = 0;
				}
			} else {

			}

		}

		if ((HAL_GetTick() > time3) && 1) {	//10s smyka, kvli watchdogu v BQ24725A
			time3 = HAL_GetTick() + 10000;

			if (chg_enable) {
				SMBus_Write_Word(BQ24725A_Address, BQ24725A_ChargeCurrent, memChargeCurrent);
				SMBus_Write_Word(BQ24725A_Address, BQ24725A_ChargeVoltage, memChargeVoltage);
				timer24h++;
			} else {
				SMBus_Write_Word(BQ24725A_Address, BQ24725A_ChargeCurrent, 0);
				SMBus_Write_Word(BQ24725A_Address, BQ24725A_ChargeVoltage, 0);
			}

		}

		if ((HAL_GetTick() > time4) && memBatteryPeriod && 1) {	//nastaviteln smyka na odesln zprv,
			//pokud memBatteryPeriod = 0 tak se neprovede
			time4 = HAL_GetTick() + memBatteryPeriod * 100;	//nastaviteln v krocch 100 ms
			//seskladani zpravy s Voltage, Current a RSOC
			ReadWordBuf[0] = SMBus_Read_Word(BQ40Z80_Address, BQ40Z80_Voltage);
			ReadWordBuf[1] = SMBus_Read_Word(BQ40Z80_Address, BQ40Z80_Current);
			ReadWordBuf[2] = SMBus_Read_Word(BQ40Z80_Address, BQ40Z80_RSOC);
			CAN_Tx_Buffer[0] = (uint8_t) (ReadWordBuf[0]);
			CAN_Tx_Buffer[1] = (uint8_t) (ReadWordBuf[0] >> 8);
			CAN_Tx_Buffer[2] = (uint8_t) (ReadWordBuf[1]);
			CAN_Tx_Buffer[3] = (uint8_t) (ReadWordBuf[1] >> 8);
			CAN_Tx_Buffer[4] = (uint8_t) (ReadWordBuf[2]);
			CAN_Send_Message(ID_Battery_data, 5, &CAN_Tx_Buffer[0]);

		}

		if (HAL_CAN_GetRxFifoFillLevel(&hcan1, CAN_RX_FIFO0) > 0) { //je neco ve fifu, prisla zprava po canu
			//zavolat can handle
			CAN_msg_sort();

		}

		if (!HAL_GPIO_ReadPin(B1_GPIO_Port, B1_Pin)) {	//LOW je zmackle TLACITKO
			if (buttonblok) {
				buttonblok = 0; //prostor pro kod

				if (onetimeonly) {
					chargeoption = SMBus_Read_Word(BQ24725A_Address, BQ24725A_ChargeOption);	//nacist charge option z BQ24725A
					SMBus_Write_Word(BQ24725A_Address, BQ24725A_ChargeOption, chargeoption ^ charge_option_charge_inhibit);
					//znegovat bit inhibit charge - zapnout/vypnout dobijeni tlacitkem

				}

				//BQ24725A_test();
				//BQ40Z80_test();

				/*
				 SMBus_Write_Word(BQ24725A_Address, BQ24725A_InputCurrent, 0x1F80);		//8064mA input current
				 SMBus_Write_Word(BQ24725A_Address, BQ24725A_ChargeVoltage, 0x3130);	//12592mV charging voltage
				 //SMBus_Write_Word(BQ24725A_Address, BQ24725A_ChargeCurrent, 0x0800);	//2048mA charging current
				 SMBus_Write_Word(BQ24725A_Address, BQ24725A_ChargeCurrent, 0x1800);	//8128mA charging current
				 SMBus_Write_Word(BQ24725A_Address, BQ24725A_ChargeOption, 0xF902);	//enable charging
				 //HAL_Delay(10000);
				 proudec = SMBus_Read_Word(BQ40Z80_Address, 0x0A); //current
				 //SMBus_Write_Word(BQ24725A_Address, BQ24725A_ChargeOption, 0xF903);
				 */
			}
		} else {
			buttonblok = 1;
		}

		/* USER CODE END WHILE */

		/* USER CODE BEGIN 3 */
	}
	/* USER CODE END 3 */
}

/**
 * @brief System Clock Configuration
 * @retval None
 */
void SystemClock_Config(void) {
	RCC_OscInitTypeDef RCC_OscInitStruct = { 0 };
	RCC_ClkInitTypeDef RCC_ClkInitStruct = { 0 };
	RCC_PeriphCLKInitTypeDef PeriphClkInit = { 0 };

	/** Initializes the RCC Oscillators according to the specified parameters
	 * in the RCC_OscInitTypeDef structure.
	 */
	RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSI;
	RCC_OscInitStruct.HSIState = RCC_HSI_ON;
	RCC_OscInitStruct.HSICalibrationValue = RCC_HSICALIBRATION_DEFAULT;
	RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
	RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSI;
	RCC_OscInitStruct.PLL.PLLM = 1;
	RCC_OscInitStruct.PLL.PLLN = 10;
	RCC_OscInitStruct.PLL.PLLP = RCC_PLLP_DIV7;
	RCC_OscInitStruct.PLL.PLLQ = RCC_PLLQ_DIV2;
	RCC_OscInitStruct.PLL.PLLR = RCC_PLLR_DIV2;
	if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK) {
		Error_Handler();
	}
	/** Initializes the CPU, AHB and APB buses clocks
	 */
	RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK | RCC_CLOCKTYPE_SYSCLK | RCC_CLOCKTYPE_PCLK1 | RCC_CLOCKTYPE_PCLK2;
	RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
	RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
	RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV1;
	RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;

	if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_4) != HAL_OK) {
		Error_Handler();
	}
	PeriphClkInit.PeriphClockSelection = RCC_PERIPHCLK_USART2 | RCC_PERIPHCLK_I2C1 | RCC_PERIPHCLK_ADC;
	PeriphClkInit.Usart2ClockSelection = RCC_USART2CLKSOURCE_PCLK1;
	PeriphClkInit.I2c1ClockSelection = RCC_I2C1CLKSOURCE_PCLK1;
	PeriphClkInit.AdcClockSelection = RCC_ADCCLKSOURCE_PLLSAI1;
	PeriphClkInit.PLLSAI1.PLLSAI1Source = RCC_PLLSOURCE_HSI;
	PeriphClkInit.PLLSAI1.PLLSAI1M = 1;
	PeriphClkInit.PLLSAI1.PLLSAI1N = 8;
	PeriphClkInit.PLLSAI1.PLLSAI1P = RCC_PLLP_DIV7;
	PeriphClkInit.PLLSAI1.PLLSAI1Q = RCC_PLLQ_DIV2;
	PeriphClkInit.PLLSAI1.PLLSAI1R = RCC_PLLR_DIV2;
	PeriphClkInit.PLLSAI1.PLLSAI1ClockOut = RCC_PLLSAI1_ADC1CLK;
	if (HAL_RCCEx_PeriphCLKConfig(&PeriphClkInit) != HAL_OK) {
		Error_Handler();
	}
	/** Configure the main internal regulator output voltage
	 */
	if (HAL_PWREx_ControlVoltageScaling(PWR_REGULATOR_VOLTAGE_SCALE1) != HAL_OK) {
		Error_Handler();
	}
}

/**
 * @brief ADC1 Initialization Function
 * @param None
 * @retval None
 */
static void MX_ADC1_Init(void) {

	/* USER CODE BEGIN ADC1_Init 0 */

	/* USER CODE END ADC1_Init 0 */

	ADC_MultiModeTypeDef multimode = { 0 };
	ADC_ChannelConfTypeDef sConfig = { 0 };

	/* USER CODE BEGIN ADC1_Init 1 */

	/* USER CODE END ADC1_Init 1 */
	/** Common config
	 */
	hadc1.Instance = ADC1;
	hadc1.Init.ClockPrescaler = ADC_CLOCK_ASYNC_DIV1;
	hadc1.Init.Resolution = ADC_RESOLUTION_12B;
	hadc1.Init.DataAlign = ADC_DATAALIGN_RIGHT;
	hadc1.Init.ScanConvMode = ADC_SCAN_DISABLE;
	hadc1.Init.EOCSelection = ADC_EOC_SINGLE_CONV;
	hadc1.Init.LowPowerAutoWait = DISABLE;
	hadc1.Init.ContinuousConvMode = ENABLE;
	hadc1.Init.NbrOfConversion = 1;
	hadc1.Init.DiscontinuousConvMode = DISABLE;
	hadc1.Init.ExternalTrigConv = ADC_SOFTWARE_START;
	hadc1.Init.ExternalTrigConvEdge = ADC_EXTERNALTRIGCONVEDGE_NONE;
	hadc1.Init.DMAContinuousRequests = DISABLE;
	hadc1.Init.Overrun = ADC_OVR_DATA_PRESERVED;
	hadc1.Init.OversamplingMode = DISABLE;
	if (HAL_ADC_Init(&hadc1) != HAL_OK) {
		Error_Handler();
	}
	/** Configure the ADC multi-mode
	 */
	multimode.Mode = ADC_MODE_INDEPENDENT;
	if (HAL_ADCEx_MultiModeConfigChannel(&hadc1, &multimode) != HAL_OK) {
		Error_Handler();
	}
	/** Configure Regular Channel
	 */
	sConfig.Channel = ADC_CHANNEL_3;
	sConfig.Rank = ADC_REGULAR_RANK_1;
	sConfig.SamplingTime = ADC_SAMPLETIME_640CYCLES_5;
	sConfig.SingleDiff = ADC_SINGLE_ENDED;
	sConfig.OffsetNumber = ADC_OFFSET_NONE;
	sConfig.Offset = 0;
	if (HAL_ADC_ConfigChannel(&hadc1, &sConfig) != HAL_OK) {
		Error_Handler();
	}
	/* USER CODE BEGIN ADC1_Init 2 */

	/* USER CODE END ADC1_Init 2 */

}

/**
 * @brief ADC2 Initialization Function
 * @param None
 * @retval None
 */
static void MX_ADC2_Init(void) {

	/* USER CODE BEGIN ADC2_Init 0 */

	/* USER CODE END ADC2_Init 0 */

	ADC_ChannelConfTypeDef sConfig = { 0 };

	/* USER CODE BEGIN ADC2_Init 1 */

	/* USER CODE END ADC2_Init 1 */
	/** Common config
	 */
	hadc2.Instance = ADC2;
	hadc2.Init.ClockPrescaler = ADC_CLOCK_ASYNC_DIV1;
	hadc2.Init.Resolution = ADC_RESOLUTION_12B;
	hadc2.Init.DataAlign = ADC_DATAALIGN_RIGHT;
	hadc2.Init.ScanConvMode = ADC_SCAN_DISABLE;
	hadc2.Init.EOCSelection = ADC_EOC_SINGLE_CONV;
	hadc2.Init.LowPowerAutoWait = DISABLE;
	hadc2.Init.ContinuousConvMode = ENABLE;
	hadc2.Init.NbrOfConversion = 1;
	hadc2.Init.DiscontinuousConvMode = DISABLE;
	hadc2.Init.ExternalTrigConv = ADC_SOFTWARE_START;
	hadc2.Init.ExternalTrigConvEdge = ADC_EXTERNALTRIGCONVEDGE_NONE;
	hadc2.Init.DMAContinuousRequests = DISABLE;
	hadc2.Init.Overrun = ADC_OVR_DATA_PRESERVED;
	hadc2.Init.OversamplingMode = DISABLE;
	if (HAL_ADC_Init(&hadc2) != HAL_OK) {
		Error_Handler();
	}
	/** Configure Regular Channel
	 */
	sConfig.Channel = ADC_CHANNEL_4;
	sConfig.Rank = ADC_REGULAR_RANK_1;
	sConfig.SamplingTime = ADC_SAMPLETIME_640CYCLES_5;
	sConfig.SingleDiff = ADC_SINGLE_ENDED;
	sConfig.OffsetNumber = ADC_OFFSET_NONE;
	sConfig.Offset = 0;
	if (HAL_ADC_ConfigChannel(&hadc2, &sConfig) != HAL_OK) {
		Error_Handler();
	}
	/* USER CODE BEGIN ADC2_Init 2 */

	/* USER CODE END ADC2_Init 2 */

}

/**
 * @brief CAN1 Initialization Function
 * @param None
 * @retval None
 */
static void MX_CAN1_Init(void) {

	/* USER CODE BEGIN CAN1_Init 0 */

	/* USER CODE END CAN1_Init 0 */

	/* USER CODE BEGIN CAN1_Init 1 */

	/* USER CODE END CAN1_Init 1 */
	hcan1.Instance = CAN1;
	hcan1.Init.Prescaler = 80;
	hcan1.Init.Mode = CAN_MODE_NORMAL;
	hcan1.Init.SyncJumpWidth = CAN_SJW_1TQ;
	hcan1.Init.TimeSeg1 = CAN_BS1_5TQ;
	hcan1.Init.TimeSeg2 = CAN_BS2_4TQ;
	hcan1.Init.TimeTriggeredMode = DISABLE;
	hcan1.Init.AutoBusOff = DISABLE;
	hcan1.Init.AutoWakeUp = DISABLE;
	hcan1.Init.AutoRetransmission = ENABLE;
	hcan1.Init.ReceiveFifoLocked = DISABLE;
	hcan1.Init.TransmitFifoPriority = DISABLE;
	if (HAL_CAN_Init(&hcan1) != HAL_OK) {
		Error_Handler();
	}
	/* USER CODE BEGIN CAN1_Init 2 */

	/* USER CODE END CAN1_Init 2 */

}

/**
 * @brief I2C1 Initialization Function
 * @param None
 * @retval None
 */
static void MX_I2C1_Init(void) {

	/* USER CODE BEGIN I2C1_Init 0 */

	/* USER CODE END I2C1_Init 0 */

	/* USER CODE BEGIN I2C1_Init 1 */

	/* USER CODE END I2C1_Init 1 */
	hi2c1.Instance = I2C1;
	hi2c1.Init.Timing = 0x10909CEC;
	hi2c1.Init.OwnAddress1 = 0;
	hi2c1.Init.AddressingMode = I2C_ADDRESSINGMODE_7BIT;
	hi2c1.Init.DualAddressMode = I2C_DUALADDRESS_DISABLE;
	hi2c1.Init.OwnAddress2 = 0;
	hi2c1.Init.OwnAddress2Masks = I2C_OA2_NOMASK;
	hi2c1.Init.GeneralCallMode = I2C_GENERALCALL_DISABLE;
	hi2c1.Init.NoStretchMode = I2C_NOSTRETCH_DISABLE;
	if (HAL_I2C_Init(&hi2c1) != HAL_OK) {
		Error_Handler();
	}
	/** Configure Analogue filter
	 */
	if (HAL_I2CEx_ConfigAnalogFilter(&hi2c1, I2C_ANALOGFILTER_ENABLE) != HAL_OK) {
		Error_Handler();
	}
	/** Configure Digital filter
	 */
	if (HAL_I2CEx_ConfigDigitalFilter(&hi2c1, 0) != HAL_OK) {
		Error_Handler();
	}
	/* USER CODE BEGIN I2C1_Init 2 */

	/* USER CODE END I2C1_Init 2 */

}

/**
 * @brief USART2 Initialization Function
 * @param None
 * @retval None
 */
static void MX_USART2_UART_Init(void) {

	/* USER CODE BEGIN USART2_Init 0 */

	/* USER CODE END USART2_Init 0 */

	/* USER CODE BEGIN USART2_Init 1 */

	/* USER CODE END USART2_Init 1 */
	huart2.Instance = USART2;
	huart2.Init.BaudRate = 115200;
	huart2.Init.WordLength = UART_WORDLENGTH_8B;
	huart2.Init.StopBits = UART_STOPBITS_1;
	huart2.Init.Parity = UART_PARITY_NONE;
	huart2.Init.Mode = UART_MODE_TX_RX;
	huart2.Init.HwFlowCtl = UART_HWCONTROL_NONE;
	huart2.Init.OverSampling = UART_OVERSAMPLING_16;
	huart2.Init.OneBitSampling = UART_ONE_BIT_SAMPLE_DISABLE;
	huart2.AdvancedInit.AdvFeatureInit = UART_ADVFEATURE_NO_INIT;
	if (HAL_UART_Init(&huart2) != HAL_OK) {
		Error_Handler();
	}
	/* USER CODE BEGIN USART2_Init 2 */

	/* USER CODE END USART2_Init 2 */

}

/**
 * @brief GPIO Initialization Function
 * @param None
 * @retval None
 */
static void MX_GPIO_Init(void) {
	GPIO_InitTypeDef GPIO_InitStruct = { 0 };

	/* GPIO Ports Clock Enable */
	__HAL_RCC_GPIOC_CLK_ENABLE()
	;
	__HAL_RCC_GPIOH_CLK_ENABLE()
	;
	__HAL_RCC_GPIOA_CLK_ENABLE()
	;
	__HAL_RCC_GPIOB_CLK_ENABLE()
	;

	/*Configure GPIO pin Output Level */
	HAL_GPIO_WritePin(GPIOA, LD2_Pin | CAN_EN_Pin | RS_Pin, GPIO_PIN_RESET);

	/*Configure GPIO pin Output Level */
	HAL_GPIO_WritePin(EN_GPIO_Port, EN_Pin, GPIO_PIN_RESET);

	/*Configure GPIO pin : B1_Pin */
	GPIO_InitStruct.Pin = B1_Pin;
	GPIO_InitStruct.Mode = GPIO_MODE_IT_FALLING;
	GPIO_InitStruct.Pull = GPIO_NOPULL;
	HAL_GPIO_Init(B1_GPIO_Port, &GPIO_InitStruct);

	/*Configure GPIO pins : LD2_Pin CAN_EN_Pin RS_Pin */
	GPIO_InitStruct.Pin = LD2_Pin | CAN_EN_Pin | RS_Pin;
	GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
	GPIO_InitStruct.Pull = GPIO_NOPULL;
	GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
	HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);

	/*Configure GPIO pin : ACOK_Pin */
	GPIO_InitStruct.Pin = ACOK_Pin;
	GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
	GPIO_InitStruct.Pull = GPIO_NOPULL;
	HAL_GPIO_Init(ACOK_GPIO_Port, &GPIO_InitStruct);

	/*Configure GPIO pin : EN_Pin */
	GPIO_InitStruct.Pin = EN_Pin;
	GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
	GPIO_InitStruct.Pull = GPIO_NOPULL;
	GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
	HAL_GPIO_Init(EN_GPIO_Port, &GPIO_InitStruct);

}

/* USER CODE BEGIN 4 */

/* USER CODE END 4 */

/**
 * @brief  This function is executed in case of error occurrence.
 * @retval None
 */
void Error_Handler(void) {
	/* USER CODE BEGIN Error_Handler_Debug */
	/* User can add his own implementation to report the HAL error return state */
	__disable_irq();
	while (1) {
	}
	/* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
 * @brief  Reports the name of the source file and the source line number
 *         where the assert_param error has occurred.
 * @param  file: pointer to the source file name
 * @param  line: assert_param error line source number
 * @retval None
 */
void assert_failed(uint8_t *file, uint32_t line)
{
	/* USER CODE BEGIN 6 */
	/* User can add his own implementation to report the file name and line number,
	 ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
	/* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
